/* Andrey N. Kuznetsov, Almaty, 2012 *
 * For educational purposes.         *
 * Under GPL 2.0                     */
//---------------------------------------------------------------------
#include "TCentralComponent.h"
//---------------------------------------------------------------------
#define tr(s) String::fromUTF8(s)
//---------------------------------------------------------------------
TCentralComponent::TCentralComponent() : Component("Central Component"),
                                         pTextEditor(0),
                                         pAppendTextEdit(0),
                                         pFontSelectionBox(0),
                                         pClearButton(0),
                                         pAppendTextButton(0),
                                         pSetTextButton(0),
                                         pSeparator(0),
                                         pLabel(0),
                                         pLabelGroup(0),
                                         pReadOnlyButton(0),
                                         pEditByOneClickButton(0),
                                         pEditByDoubleClickButton(0),
                                         pSeparator1(0)
{
    // Создаём новое поле ввода (текстовый редактор)
    pTextEditor = new TextEditor("TextEditor");
    // Разрешаем ввод более одной строки
    // и разрешаем автоматический перенос строк
    pTextEditor->setMultiLine(true, true);
    // Запрет на редактирование
    pTextEditor->setReadOnly(true);
    // Показываем полосы прокрутки
    pTextEditor->setScrollbarsShown(true);
    addAndMakeVisible(pTextEditor);

    // Шрифт для ранее созданного поля ввода
    Font TextEditorFont;
    // Гарнитура - шрифт Sans-Serif по умолчанию
    TextEditorFont.setTypefaceName(Font::getDefaultSerifFontName());
    TextEditorFont.setHeight(20);
    // Устанавливаем шрифт TextEditorFont в качестве
    // шрифта поля ввода
    pTextEditor->setFont(TextEditorFont);
    pTextEditor->setTextToShowWhenEmpty("Text Editor", Colours::grey);

    // Создаём поле ввода для текста, добавляемого
    // в многострочное поле pTextEditor
    pAppendTextEdit = new TextEditor("AppendTextEdit");
    // Ввод текста - в одну строку
    pAppendTextEdit->setMultiLine(false);
    // Разрешаем ввод текста
    pAppendTextEdit->setReadOnly(false);
    // Показываем полосы прокрутки
    pAppendTextEdit->setScrollbarsShown(false);
    // Делаем видимым курсор
    pAppendTextEdit->setCaretVisible(true);
    // Разрешаем стандартное контекстное меню
    pAppendTextEdit->setPopupMenuEnabled(true);
    // Текст поля - пустая строка
    pAppendTextEdit->setText(String::empty);
    addAndMakeVisible(pAppendTextEdit);

    // Выпадающий список выбора шрифта
    pFontSelectionBox = new ComboBox("FontSelectionBox");
    pFontSelectionBox->setEditableText(false);
    pFontSelectionBox->setJustificationType(Justification::centredLeft);
    pFontSelectionBox->setTextWhenNothingSelected(tr("Выбрать"));
    // Устанавливаем в качестве слушателя выпадающего списка
    // сам компонент-контейнер
    pFontSelectionBox->addListener(this);
    addAndMakeVisible(pFontSelectionBox);

    // Получаем имена системных шрифтов, сохраняем их 
    // в строковом массиве...
    StringArray sSystemFonts = Font::findAllTypefaceNames();
    // и добавляем в выпадающий список
    for(int i = 0; i < sSystemFonts.size(); i++)
    {
        pFontSelectionBox->addItem(sSystemFonts[i], i + 1);
    }

    // Кнопка, очищающая поле pTextEditor
    pClearButton = new TextButton("ClearButton");
    pClearButton->setButtonText(tr("Очистить"));
    // Устанавливаем в качестве слушателя кнопки
    // сам компонент-контейнер
    pClearButton->addListener(this);
    addAndMakeVisible(pClearButton);

    // Кнопка, добавляющая текст в поле pTextEditor
    pAppendTextButton = new TextButton("AppendTextButton");
    pAppendTextButton->setButtonText(tr("Добавить текст"));
    pAppendTextButton->addListener(this);
    addAndMakeVisible(pAppendTextButton);

    // Кнопка, замещающая текст в поле pTextEditor
    pSetTextButton = new TextButton("SetTextButton");
    pSetTextButton->setButtonText(tr("Заменить текст"));
    pSetTextButton->addListener(this);
    addAndMakeVisible(pSetTextButton);

    // Первый разделитель
    pSeparator = new Label("Separator", String::empty);
    pSeparator->setEditable(false, false, false);
    pSeparator->setColour(Label::backgroundColourId, Colour(0x3d000000));
    addAndMakeVisible(pSeparator);

    // Ярлык с возможностью ввода текста
    pLabel = new Label("Label", "Label");
    pLabel->setFont(Font(15.0000f, Font::bold));
    pLabel->setJustificationType(Justification::centred);
    // Запрет на редактирование
    pLabel->setEditable(false, false, false);
    // Цвет текста надписи
    pLabel->setColour(Label::textColourId, Colour(0xff0f1465));
    // Цвет рамки
    pLabel->setColour(Label::outlineColourId, Colour(0xf6615e5e));
    pLabel->setColour(Label::backgroundColourId, Colour(0x0));
    addAndMakeVisible(pLabel);

    pLabelGroup = new GroupComponent();
    pLabelGroup->setText(tr("Редактирование"));
    addAndMakeVisible(pLabelGroup);

    // Радиокнопка - запрет на редактирование ярлыка
    pReadOnlyButton = new ToggleButton("ReadOnlyButton");
    pReadOnlyButton->setButtonText(tr("Только чтение"));
    // Устанавливаем в качестве слушателя кнопки
    // сам компонент-контейнер
    pReadOnlyButton->addListener(this);
    // Радиокнопка отмечена
    pReadOnlyButton->setToggleState(true, false);
    pReadOnlyButton->setRadioGroupId(1234);
    addAndMakeVisible(pReadOnlyButton);

    // Радиокнопка - редактирование ярлыка по щелчку
    pEditByOneClickButton = new ToggleButton("EditByOneClickButton");
    pEditByOneClickButton->setButtonText(tr("Одним щелчком"));
    pEditByOneClickButton->addListener(this);
    // Радиокнопка не отмечена
    pEditByOneClickButton->setToggleState(false, false);
    pEditByOneClickButton->setRadioGroupId(1234);
    addAndMakeVisible(pEditByOneClickButton);

    // Радиокнопка - редактирование ярлыка по двойному щелчку
    pEditByDoubleClickButton = new ToggleButton("EditByDoubleClickButton");
    pEditByDoubleClickButton->setButtonText(tr("Двойным щелчком"));
    pEditByDoubleClickButton->addListener(this);
    pEditByDoubleClickButton->setToggleState(false, false);
    pEditByDoubleClickButton->setRadioGroupId(1234);
    addAndMakeVisible(pEditByDoubleClickButton);

    // Второй разделитель
    pSeparator1 = new Label("Separator1", String::empty);
    pSeparator1->setEditable(false, false, false);
    pSeparator1->setColour(Label::backgroundColourId, Colour(0x3d000000));
    addAndMakeVisible(pSeparator1);

    pPasswordLabel = new Label("PasswordLabel", tr("Введите пароль: "));
    pPasswordLabel->setEditable(false, false, false);
    addAndMakeVisible(pPasswordLabel);

    // Поле ввода пароля
    pPasswordEdit = new TextEditor("PasswordEdit", 0x2022);
    // Разрешаем ввод цифр; ограничений на длину вводимой строки нет
    pPasswordEdit->setInputRestrictions(0, "1234567890");
    pPasswordEdit->setMultiLine(false);
    pPasswordEdit->setScrollbarsShown(false);
    pPasswordEdit->setReturnKeyStartsNewLine(false);
    pPasswordEdit->addKeyListener(this);
    addAndMakeVisible(pPasswordEdit);

    // Кнопка ввода пароля
    pSubmitButton = new TextButton("SubmitButton");
    pSubmitButton->setButtonText(tr("Ввод"));
    pSubmitButton->addListener(this);
    addAndMakeVisible(pSubmitButton);

    setSize(728, 400);

    // Задаём пароль
    sPassword = "1234";
}
//---------------------------------------------------------------------
TCentralComponent::~TCentralComponent()
{
    // Удаляем дочерние виджеты
    // и обнуляем их указатели
    deleteAllChildren();
}
//---------------------------------------------------------------------
void TCentralComponent::paint(Graphics& Canvas)
{
    Canvas.fillAll(Colours::azure);
}
//---------------------------------------------------------------------
void TCentralComponent::resized()
{
    pTextEditor->setBounds(8, 8, 416, 120);
    pAppendTextEdit->setBounds(432, 8, 152, 24);
    pFontSelectionBox->setBounds(592, 8, 112, 24);
    pClearButton->setBounds(432, 40, 150, 24);
    pAppendTextButton->setBounds(432, 72, 150, 24);
    pSetTextButton->setBounds(432, 104, 150, 24);
    pSeparator->setBounds(8, 136, 704, 1);
    pLabel->setBounds(8, 144, 416, 24);
    pLabelGroup->setBounds(432, 144, 272, 136);
    pReadOnlyButton->setBounds(448, 168, 150, 24);
    pEditByOneClickButton->setBounds(448, 200, 150, 24);
    pEditByDoubleClickButton->setBounds(448, 232, 150, 24);
    pSeparator1->setBounds(8, 288, 704, 1);
    pPasswordLabel->setBounds(8, 296, 216, 24);
    pPasswordEdit->setBounds(232, 296, 192, 24);
    pSubmitButton->setBounds(432, 296, 150, 24);
}
//---------------------------------------------------------------------
void TCentralComponent::comboBoxChanged(ComboBox* pComboBox)
{
    // Изменён выпадающий список гарнитуры шрифта
    if(pComboBox == pFontSelectionBox)
    {
        // Устанавливаем в качестве шрифта текста поля ввода
        // шрифт, выбранный пользователем из списка
        Font CurrentFont;
        CurrentFont.setTypefaceName(pFontSelectionBox->getText());
        CurrentFont.setHeight(20);
        // Задаём шрифт для отображения текста
        pTextEditor->applyFontToAllText(CurrentFont);
    }
}
//---------------------------------------------------------------------
void TCentralComponent::buttonClicked(Button* pButton)
{
    // Если нажата кнопка "Очистить"...
    if(pButton == pClearButton)
    {
        // Очистить поле ввода
        pTextEditor->clear();
    }
    // Если нажата кнопка "Добавить текст"...
    else if(pButton == pAppendTextButton)
    {
        // Вставить текст за курсором
        pTextEditor->insertTextAtCaret(pAppendTextEdit->getText());
        pTextEditor->insertTextAtCaret("\n");
    }
    // Если нажата кнопка "Заменить текст"...
    else if(pButton == pSetTextButton)
    {
        // Сделать замену
        pTextEditor->setText(pAppendTextEdit->getText());
        pTextEditor->insertTextAtCaret("\n");
    }
    else if(pButton == pReadOnlyButton)
    {
        pLabel->setEditable(false, false, false);
    }
    else if(pButton == pEditByOneClickButton)
    {
        pLabel->setEditable(true, false, false);
    }
    else if(pButton == pEditByDoubleClickButton)
    {
        pLabel->setEditable(false, true, false);
    }
    else if(pButton == pSubmitButton)
    {
        SubmitPassword();
    }
}
//---------------------------------------------------------------------
void TCentralComponent::SubmitPassword()
{
    // Если введённый текст не содержит пустую строку
    // или строку, заполненную пробелами
    if(pPasswordEdit->getText().containsNonWhitespaceChars())
    {
        // И если пароль соответствует введённому тексту
        if(pPasswordEdit->getText() == sPassword)
        {
            // Выводим информационное сообщение
            pPasswordLabel->setColour(Label::textColourId, Colours::blue);
            pPasswordLabel->setText(tr("Принято!"), false);
            // Делаем недоступными кнопку и поле ввода
            pPasswordEdit->setEnabled(false);
            pSubmitButton->setEnabled(false);
        }
        else
        {
            pPasswordLabel->setColour(Label::textColourId, Colours::red);
            pPasswordLabel->setText(tr("Пароль неверен! Введите повторно:"), false);
            pPasswordEdit->clear();
        }
    }
}
//---------------------------------------------------------------------
bool TCentralComponent::keyPressed(const KeyPress& Key, Component* pComponent)
{
    // Если в фокусе другое поле ввода - выходим
    if(pComponent != pPasswordEdit) return false;
    // Если нажата не клавиша <ENTER> - выходим
    else if(Key != KeyPress::returnKey) return false;
    else SubmitPassword();
    return true;
}
//---------------------------------------------------------------------
